{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE RecordWildCards #-}

module Kululaskusilppuri.Types (Kululasku(..)) where

import Data.ByteString.Lazy (ByteString)
import Data.Maybe (listToMaybe, fromMaybe)

import qualified Data.ByteString as BS
import qualified Data.ByteString.Lazy.Char8 as LC8
import Data.Text.Lazy.Encoding as LTE
import Data.Text.Encoding as TE

import Data.Csv

data Kululasku = Kululasku
    { referenceNumber :: ByteString
    , submitter :: ByteString
    , amount :: ByteString
    , description :: ByteString
    , paymentDate :: ByteString
    } deriving (Show, Eq)

instance FromNamedRecord Kululasku where
    parseNamedRecord row = Kululasku
        <$> row .: (TE.encodeUtf8 "Kululaskun viitenumero")
        <*> row .: (TE.encodeUtf8 "Tekijä")
        <*> row .: (TE.encodeUtf8 "Rivin summa")
        <*> (constructDescription <$> row .: (TE.encodeUtf8 "Rivin luokka") <*> row .: (TE.encodeUtf8 "Rivin kuvaus"))
        <*> (timeToDate <$> row .: (TE.encodeUtf8 "Maksettu"))
            where timeToDate = fromMaybe "" . listToMaybe . LC8.words
                  -- "YYYY-MM-DD HH:MM:SS" -> "YYYY-MM-DD"
                  constructDescription classification description = classification <> (LTE.encodeUtf8 " / ") <> description

instance ToNamedRecord Kululasku where
    toNamedRecord Kululasku {..} = namedRecord
        [ referenceField .= referenceNumber 
        , submitterField .= submitter
        , amountField .= amount
        , descriptionField .= description
        , dateField .= paymentDate
        ]

instance DefaultOrdered Kululasku where
    headerOrder _ = header [dateField, referenceField, submitterField, amountField, descriptionField]

dateField, referenceField, submitterField, amountField, descriptionField :: BS.ByteString
dateField = TE.encodeUtf8 "Päivämäärä"
referenceField = TE.encodeUtf8 "Viitenumero"
submitterField = TE.encodeUtf8 "Saaja"
amountField = TE.encodeUtf8 "Summa"
descriptionField = TE.encodeUtf8 "Selite"
